/*
Dozen Days of Words episode 7: Crossword
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package
{
	import flash.display.MovieClip;
	import flash.events.*;
	import flash.text.TextField;
	import flash.ui.Keyboard;

	import com.blazinggames.dozendays.*;
	import com.blazinggames.dozendays.words.*;
	import com.blazinggames.dozendays.words.crossword.*;

	[SWF(width="640", height="480", backgroundColor="#000000")]
	
	public class Crossword extends MovieClip implements IGameMode
	{
// ***** CONSTANTS ***********************************************************
		public static const PAGE1_TEXT:String = 
"This game was created for Blazing Games Inc. by Billy D. Spelchan "+
"(the person writing this).\n\n"+
"This game is part of the Dozen Days of Words series which is a collection of "+
"Games that are created in under 24 hours of working time that revolve around "+
"the creation of computer versions of word games. This episode is the word "+
"game that pretty much everybody is familiar with. Crossword puzzles are "+
"puzzles where players have to fill in a grid of horizontal and vertical "+
"letters with the appropriate words taking into account the clues that are "+
"provided.";

		public static const PAGE2_TEXT:String = 
"This program is free software; you can redistribute it and/or modify "+
"it under the terms of the GNU General Public License as published by "+
"the Free Software Foundation; either version 2 of the License, or "+
"(at your option) any later version.\n\n"+
"This program is distributed in the hope that it will be useful, "+
"but WITHOUT ANY WARRANTY; without even the implied warranty of "+
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "+
"GNU General Public License for more details.\n\n"+
"You should have received a copy of the GNU General Public License "+
"along with this program; if not, write to the Free Software "+
"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA";

		public static const MODE_NONE:int = 0;
		public static const MODE_TITLE:int = 1;
		public static const MODE_GAME:int = 2;
		public static const MODE_ABOUT:int = 3;

		public static const PUZZLE:XML = 
<DDW7_CROSSWORD>
	<BOARDSIZE width='16' height='16'/>
	<HORIZONTAL x='0' y='2' word='ROULETTE'>A glitzy casino game featuring a spinning wheel and a ball.</HORIZONTAL>
	<HORIZONTAL x='9' y='2' word='NUCLEAR'>The type of bomb in the retro 3D search game.</HORIZONTAL>
	<HORIZONTAL x='5' y='4' word='COFFEE'>The name of the quest to make a certain drink.</HORIZONTAL>
	<HORIZONTAL x='6' y='5' word='GL'>The name of the Coffee Quest game that uses accelerated 3D graphics.</HORIZONTAL>
	<HORIZONTAL x='0' y='8' word='BACCARAT'>The American version of Chemin De Fer.</HORIZONTAL>
	<HORIZONTAL x='9' y='9' word='HANGMAN'>A game where you have to spell a secret word.</HORIZONTAL>
	<HORIZONTAL x='7' y='10' word='DARTS'>A game that consists of 301, 501, 1001, and Around the Clock variants.</HORIZONTAL>
	<HORIZONTAL x='1' y='11' word='SOLITAIRE'>American patience.</HORIZONTAL>
	<HORIZONTAL x='6' y='12' word='PACE'>The speed you travel.</HORIZONTAL>
	<HORIZONTAL x='4' y='13' word='EARL'>A title for royalty.</HORIZONTAL>
	<HORIZONTAL x='9' y='13' word='SCRATCH'>Luck and Halloween are my variants of this game.</HORIZONTAL>
	<HORIZONTAL x='6' y='15' word='LEAP'>What you do in my February 29th game.</HORIZONTAL>

	<VERTICAL x='1' y='1' word='DOORS'>A means of entrance that can be deadly.</VERTICAL>
	<VERTICAL x='6' y='0' word='HOTDOG'>My retail establishment game revolves around these.</VERTICAL>
	<VERTICAL x='9' y='1' word='ANGER'>Pent Up x.</VERTICAL>
	<VERTICAL x='13' y='0' word='CHECKERS'>Board game involving jumping and kings.</VERTICAL>
	<VERTICAL x='15' y='2' word='ROMP'>Dungeon x.</VERTICAL>
	<VERTICAL x='7' y='4' word='FLEET'>Group of ships.</VERTICAL>
	<VERTICAL x='1' y='7' word='TANKS'>x for the Memory.</VERTICAL>
	<VERTICAL x='4' y='7' word='VAMPIRE'>x Attack.</VERTICAL>
	<VERTICAL x='9' y='8' word='THREES'>A dice game named after a number that scores 0.</VERTICAL>
	<VERTICAL x='11' y='7' word='PUNS'>The type of humor that I rely on far too often.</VERTICAL>
	<VERTICAL x='15' y='9' word='NIM'>Take the last object.</VERTICAL>
	<VERTICAL x='10' y='9' word='AT'>The sybol used in email.</VERTICAL>
	<VERTICAL x='7' y='10' word='DIAL'>A common control.</VERTICAL>
	<VERTICAL x='8' y='10' word='ARC'>A graphical curve.</VERTICAL>
	<VERTICAL x='6' y='11' word='APRIL'>The month of the fool.</VERTICAL>
</DDW7_CROSSWORD>
/*
<DDW7_CROSSWORD>
	<BOARDSIZE width='9' height='9'/>
	<HORIZONTAL x='2' y='1' word='TEST'>What you do when you are debugging</HORIZONTAL>
	<HORIZONTAL x='1' y='3' word='GAMES'>What you play for entertainment</HORIZONTAL>
	<HORIZONTAL x='1' y='5' word='BLAZING'>Burining Brightly</HORIZONTAL>
	<VERTICAL x='5' y='1' word='TESTING'>Another word for Debugging</VERTICAL>
</DDW7_CROSSWORD>
*/

// ***** VARIABLES ***********************************************************
		
		private var _mode:int;
		private var _title:CrosswordTitle;
		private var _game:CrosswordGame;
		private var _about:About;
		
// ***** CONSTRUCTOR *********************************************************

		public function Crossword()
		{
			_mode = MODE_NONE;
			_title = new CrosswordTitle(this);
			_game = new CrosswordGame(this, PUZZLE);

			_about = new About(this);
			_about.setCaption("About Word Slide");
			_about.addPage(PAGE1_TEXT);
			_about.addPage(PAGE2_TEXT);
			changeMode(MODE_TITLE);
//			stage.addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
			stop();
		}
		
// ***** Functions - mode handlers ******************************************		
		
		public function changeMode(n:int):void
		{
			// remove old mode
			switch(_mode) {
				case MODE_NONE:
					break;
				case MODE_GAME:
					removeChild(_game);
					break;
				case MODE_TITLE:
					removeChild(_title);
					break;
				case MODE_ABOUT:
					removeChild(_about);
					break;
			}
			
			// change to the new mode
			switch(n) {
				case MODE_NONE:
					break;
				case MODE_GAME:
					_game.startGame(_title.getPlayMode());
					addChildAt(_game, 0);
					break;
				case MODE_TITLE:
					addChildAt(_title, 0);
					break;
				case MODE_ABOUT:
					addChildAt(_about, 0);
					break;
			}
			
			_mode = n;
		}
/*
		public function handleEnterFrame(evt:Event):void
		{
			if (_mode == MODE_GAME)
				_game.updateTime();
		}
*/
		public function handleKeyUp(evt:KeyboardEvent):void
		{
			if (_mode != MODE_GAME)
				return;
			_game.handleKeypress(evt.keyCode);
/*
			switch (evt.keyCode) {
				case Keyboard.UP:
					_game.slideUp();
					break;
				case Keyboard.LEFT:
					_game.slideLeft();
					break;
				case Keyboard.RIGHT:
					_game.slideRight();
					break;
				case Keyboard.DOWN:
					_game.slideDown();
					break;
				
			}
*/
		}

	}
}
