/*
Dozen Days of Words episode 7: Crossword
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.crossword
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.getTimer;
	import flash.ui.Keyboard;

	import com.blazinggames.dozendays.TextButton;
	import com.blazinggames.dozendays.YesNoDialog;
	import com.blazinggames.dozendays.words.LetterTile;

	public class CrosswordGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const HINTS_OFF_TEXT:String = "Turn Hints On";
		public static const HINTS_ON_TEXT:String = "Turn Hints Off";
		public static const DONE_TEXT:String = "Exit";
		public static const EMPTY_TEXT:String = "-";
		public static const WON_TEXT:String = "You Have WON!";
		public static const LOST_TEXT:String = "You Have Lost!";

//		public static const TILE_SIZE:int = 40;
//		public static const NUM_ROWS:int = 12;
//		public static const NUM_COLS:int = 12;
		
		public static const BASE_STARTING_LETTERS:int = 5;
		public static const STARING_LETTERS_MULTIPLIER:int = 5;
		public static const POINTS_PER_LETTER:int = 5;

		public static const BACKGROUND_COLOR:uint = 0x000050;
		public static const TILE_COLOR:uint = 0x500050;
		public static const BORDER_COLOR:uint = 0x000030;
		public static const LETTER_COLOR:uint = 0xC000C0;
		public static const TILE_COLOR_DIM:uint = 0x200020;
		public static const LETTER_COLOR_DIM:uint = 0x500050;
		public static const HIGHLIGHT_COLOR:uint = 0xFF0050;
		public static const WORD_COLOR:uint = 0x00C000;

// ***** VARIABLES **********************************************************

		protected var _crossword:Crossword;
		protected var _doneButton:TextButton;
		protected var _hintsButton:TextButton;
		protected var _acrossCaption:TextField;
		protected var _acrossClue:TextField;
		protected var _downCaption:TextField;
		protected var _downClue:TextField;
		protected var _letterButtons:Array;
		protected var _tiles:Array;
		protected var _backdrop:TextField;
		protected var _haveWon:TextField;
//		protected var _scoreText:TextField;
		
		protected var _difficulty:int;
		protected var _currentLevel:int;
		protected var _score:int;
//		protected var _crosswordList:GrowingList;
		protected var _selection:LetterTile;
//		protected var _levelCompleteButton:TextButton;

		public var _boardWidth:int;
		public var _boardHeight:int;
		protected var _cursorIndex:int;
		protected var _hintsOn:Boolean;

// ***** CONSTRUCTOR ********************************************************

		public function CrosswordGame(crossword:Crossword, puzzle:XML)
		{
			var cntrRow:int, cntrCol:int, rowLength:int, tileX:int, tileY:int;
			_crossword = crossword;
			_selection = null;

			_backdrop = new TextField();
			_backdrop.multiline = true;
			_backdrop.width = 640;
			_backdrop.height = 480;
			_backdrop.textColor = 0xC000C0;
			_backdrop.background = true;
			_backdrop.backgroundColor = BACKGROUND_COLOR;
			addChild(_backdrop);
			
			_tiles = new Array();
			_boardWidth = puzzle.BOARDSIZE.@width;
			_boardHeight = puzzle.BOARDSIZE.@height;
			var tileWidth:int = 460 / _boardWidth;
			var tileHeight:int = 460 / _boardHeight;
			var indx:int = 0;
			var wordFormat:TextFormat = new TextFormat(null, 20, WORD_COLOR, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrRow = 0; cntrRow < _boardHeight; ++cntrRow) {
				tileX = 0;
				tileY = cntrRow * tileHeight;
				for (cntrCol = 0; cntrCol < _boardWidth; ++cntrCol) {
					_tiles[indx] = new CrosswordLetterTile(tileWidth,tileHeight);
					_tiles[indx].x = tileX;
					_tiles[indx].y = tileY;
//					_tiles[indx].setColors(LETTER_COLOR, TILE_COLOR);
					_tiles[indx].setBorder(true, BORDER_COLOR);
					_tiles[indx].addEventListener(LetterTile.TILE_SELECTED_EVENT, handleTileSelection);
					addChild(_tiles[indx]);

					tileX += tileWidth;
					++indx;
				}
				
			}
			_cursorIndex = 0;
			_tiles[0].setSelected(true);
			
			_letterButtons = new Array();
			var letterButtonFormat:TextFormat = new TextFormat(null, 12, 0xFFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrCol = 0; cntrCol < 26; ++cntrCol) {
				_letterButtons[cntrCol] = new TextButton(String.fromCharCode(65+cntrCol), 16, 16, letterButtonFormat);
				_letterButtons[cntrCol].x = 16 * cntrCol;
				_letterButtons[cntrCol].y = 464;
				addChild(_letterButtons[cntrCol]);
				_letterButtons[cntrCol].addEventListener(MouseEvent.MOUSE_UP, handleSelectLetter);
			}
		
			var wordData:XML
			for each (wordData in puzzle.HORIZONTAL)
				addHorizontalWord(wordData.@x, wordData.@y, wordData.@word, wordData);
			for each (wordData in puzzle.VERTICAL)
				addVerticalWord(wordData.@x, wordData.@y, wordData.@word, wordData);

			_hintsButton = new TextButton(HINTS_OFF_TEXT, 160, 40);
			_hintsButton.x = 480;
			_hintsButton.y = 380;
			addChild(_hintsButton);
			_hintsButton.addEventListener(MouseEvent.MOUSE_UP, handleHint);

			_doneButton = new TextButton(DONE_TEXT, 160, 40);
			_doneButton.x = 480;
			_doneButton.y = 430;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			var captionFormat:TextFormat = new TextFormat(null, 20, 0xF0F0FF, null, null, null, null, null, TextFormatAlign.CENTER);
			_acrossCaption = new TextField();
			_acrossCaption.width = 160;
			_acrossCaption.height = 25;
			_acrossCaption.x = 480;
			_acrossCaption.y = 0;
			_acrossCaption.defaultTextFormat = captionFormat;
			_acrossCaption.text = "Across";
			addChild(_acrossCaption);
			_downCaption = new TextField();
			_downCaption.width = 160;
			_downCaption.height = 25;
			_downCaption.x = 480;
			_downCaption.y = 175;
			_downCaption.defaultTextFormat = captionFormat;
			_downCaption.text = "Down";
			addChild(_downCaption);

			var clueFormat:TextFormat = new TextFormat(null, 16, 0xFFFFE0, null, null, null, null, null, TextFormatAlign.LEFT);
			_acrossClue = new TextField();
			_acrossClue.width = 160;
			_acrossClue.height = 150;
			_acrossClue.x = 480;
			_acrossClue.y = 25;
			_acrossClue.defaultTextFormat = clueFormat;
			_acrossClue.multiline = true;
			_acrossClue.wordWrap = true;
			_acrossClue.text = "clue goes here";
			addChild(_acrossClue);
			_downClue = new TextField();
			_downClue.width = 160;
			_downClue.height = 150;
			_downClue.x = 480;
			_downClue.y = 200;
			_downClue.defaultTextFormat = clueFormat;
			_downClue.multiline = true;
			_downClue.wordWrap = true;
			_downClue.text = "clue goes here";
			addChild(_downClue);
			
			var winFormat:TextFormat = new TextFormat(null, 90, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			_haveWon = new TextField();
			_haveWon.width = 640;
			_haveWon.height = 150;
			_haveWon.y = 200;
			_haveWon.defaultTextFormat = winFormat;
			_haveWon.text = WON_TEXT;
			_haveWon.visible = false;
			addChild(_haveWon);
		}

// ***** Functions - Puzzle Handling ****************************************

		protected function addHorizontalWord(tx:int, ty:int, word:String, clue:String):void
		{
			var indx:int = ty * _boardWidth + tx;
			for (var cntr:int = 0; cntr < word.length; ++cntr) {
				_tiles[indx].setPuzzle(word.charAt(cntr), clue, _tiles[indx].getVerticalClue());
				++indx;
			}
		}

		protected function addVerticalWord(tx:int, ty:int, word:String, clue:String):void
		{
			var indx:int = ty * _boardWidth + tx;
			for (var cntr:int = 0; cntr < word.length; ++cntr) {
				_tiles[indx].setPuzzle(word.charAt(cntr), _tiles[indx].getHorizontalClue(), clue);
				indx += _boardWidth;
			}
		}

		public function startGame(playmode:int = 3):void
		{
			
			_difficulty = playmode;
//			_score = 0;
//			_scoreText.text = "0";
			for (var cntr:int = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setLetter(EMPTY_TEXT);
			_hintsOn = false;
			_hintsButton.setLabel(HINTS_OFF_TEXT);
		}


// ***** Functions - Event Handling *****************************************

		private function handleHint(evt:Event):void
		{
			if (_hintsOn) {
				_hintsOn = false;
				_hintsButton.setLabel(HINTS_OFF_TEXT);
			} else {
				_hintsOn = true;
				_hintsButton.setLabel(HINTS_ON_TEXT);
			}
			for (var cntr:int = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setHints(_hintsOn);
		}

		private function handleExit(evt:Event):void
		{
			_crossword.changeMode(Crossword.MODE_TITLE);
		}

		private function handleSelectLetter(evt:Event):void
		{
			for (var cntr:int = 0; cntr < _letterButtons.length; ++cntr)
				if (_letterButtons[cntr] == evt.target)
					_tiles[_cursorIndex].setLetter(String.fromCharCode(65+cntr));
			if (_tiles[_cursorIndex].getHorizontalClue() != null)
				handleKeypress(Keyboard.RIGHT);
			else
				handleKeypress(Keyboard.DOWN);
		}

		private function handleTileSelection(evt:Event):void
		{
			var cntr:int;
			var selected:CrosswordLetterTile = CrosswordLetterTile(evt.target);
			var indx:int = -1;

			// figure out which tile was clicked on
			for (cntr = 0; cntr < _tiles.length; ++cntr) {
				if (selected == _tiles[cntr])
					indx = cntr;
			}
			if (indx < 0)
				return;
			
			setCursorLocation(indx);
		}

// ***** Functions - game handling *******************************************

		private function setCursorLocation(indx:int):void
		{
			_tiles[_cursorIndex].setSelected(false);
			_cursorIndex = indx;
			_tiles[_cursorIndex].setSelected(true);
			var s:String = _tiles[_cursorIndex].getHorizontalClue(); 
			if (s == null) s = "No word!"
			_acrossClue.text = s;
			s = _tiles[_cursorIndex].getVerticalClue(); 
			if (s == null) s = "No word!"
			_downClue.text = s;
			
		}

		public function handleKeypress(keycode:uint):void
		{
			if ( (keycode >= 65) && (keycode <= 90) ) {
				_tiles[_cursorIndex].setLetter(String.fromCharCode(keycode));
				if (_tiles[_cursorIndex].getHorizontalClue() != null)
					keycode = Keyboard.RIGHT;
				else
					keycode = Keyboard.DOWN;
			}
			var cursorX:int = _cursorIndex % _boardWidth;
			var cursorY:int = _cursorIndex / _boardWidth;
			switch (keycode) {
				case Keyboard.UP:
					--cursorY;
					if (cursorY < 0)
						cursorY = _boardHeight - 1;
					break;
				case Keyboard.LEFT:
					--cursorX;
					if (cursorX < 0)
						cursorX = _boardWidth - 1;
					break;
				case Keyboard.RIGHT:
					++cursorX;
					if (cursorX >= _boardWidth)
						cursorX = 0;
					break;
				case Keyboard.DOWN:
					++cursorY;
					if (cursorY >= _boardHeight)
						cursorY = 0;
					break;
			}
			setCursorLocation(cursorY * _boardWidth + cursorX);
		}

		public function getLetter(tx:int, ty:int):String
		{
			if ( (tx < 0) || (tx >= _boardWidth) || (ty < 0) || (ty >= _boardHeight) )
				return "*";
			var indx:int = ty * _boardWidth + tx;
			return _tiles[indx].getLetter();
		}

		public function setLetter(tx:int, ty:int, s:String):void
		{
			if ( (tx < 0) || (tx >= _boardWidth) || (ty < 0) || (ty >= _boardHeight) )
				return ;
			var indx:int = ty * _boardWidth + tx;
			_tiles[indx].setLetter(s);
		}
	}
}
