/*
Dozen Days of Words episode 7: Crossword
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.crossword
{
	import com.blazinggames.dozendays.words.LetterTile;

	public class CrosswordLetterTile
	extends LetterTile
	{
// ***** CONSTANTS **********************************************************

		public static const SOLID_COLOR:uint = 0x000010;
		public static const SOLID_SELECTED_COLOR:uint = 0x200040;
		public static const NORMAL_BACK_COLOR:uint = 0xC0C0FF;
		public static const NORMAL_LETTER_COLOR:uint = 0x000020;
		public static const SELECTED_BACK_COLOR:uint = 0xFFC0FF;
		public static const SELECTED_LETTER_COLOR:uint = 0x000020;
		public static const HINT_CORRECT_COLOR:uint = 0x008000;
		public static const HINT_INCORRECT_COLOR:uint = 0x800000;

// ***** VARIABLES **********************************************************

		public var _correctLetter:String;
		public var _horizontalClue:String;
		public var _verticalClue:String;
		public var _isSelected:Boolean;
		public var _areHintsOn:Boolean;

		public var _solidColor:uint;
		public var _solidSelectedColor:uint;
		public var _normalBackColor:uint;
		public var _normalLetterColor:uint;
		public var _selectedBackColor:uint;
		public var _selectedLetterColor:uint;
		public var _hintCorrectColor:uint;
		public var _hintIncorrectColor:uint;		

// ***** CONSTRUCTOR ********************************************************

		public function CrosswordLetterTile(w:int, h:int)
		{
			super(w, h, " ");
			_correctLetter = " ";
			_horizontalClue = null;
			_verticalClue = null;
			_isSelected = false;
			_areHintsOn = false;
			setGameColors();
		}

// ***** Functions - Getting and Setting**************************************

		public function setGameColors(solidColor:uint = SOLID_COLOR,
								solidSelectedColor:uint = SOLID_SELECTED_COLOR,
								normalBackColor:uint = NORMAL_BACK_COLOR,
								normalLetterColor:uint = NORMAL_LETTER_COLOR,
								selectedBackColor:uint = SELECTED_BACK_COLOR,
								selectedLetterColor:uint=SELECTED_LETTER_COLOR,
								hintCorrectColor:uint = HINT_CORRECT_COLOR,
								hintIncorrectColor:uint = HINT_INCORRECT_COLOR ):void
		{
			_solidColor = solidColor;
			_solidSelectedColor = solidSelectedColor;
			_normalBackColor = normalBackColor;
			_normalLetterColor = normalLetterColor;
			_selectedBackColor = selectedBackColor;
			_selectedLetterColor = selectedLetterColor;
			_hintCorrectColor = hintCorrectColor;
			_hintIncorrectColor = hintIncorrectColor;
			render();
		}

		public function getHorizontalClue():String
		{
			return _horizontalClue;
		}

		public function getCorrectLetter():String
		{
			return _correctLetter;
		}

		public function getVerticalClue():String
		{
			return _verticalClue;
		}

		public function setPuzzle(correctLetter:String = " ", 
									horizontalClue:String = null,
									verticalClue:String = null ):void
		{
			_correctLetter = correctLetter;
			_horizontalClue = horizontalClue;
			_verticalClue = verticalClue;
			render();
		}

		public function isSelected():Boolean
		{
			return _isSelected;
		}

		public function setSelected(b:Boolean):void
		{
			_isSelected = b;
			render();
		}

		public function areHintsOn():Boolean
		{
			return _areHintsOn;
		}

		public function setHints(b:Boolean):void
		{
			_areHintsOn = b;
			render();
		}

		override public function setLetter(s:String):void
		{
			super.setLetter(s);
			render();
		}

// ***** Functions - Rendering ***********************************************

		public function render():void
		{
			var backColor:uint = _normalBackColor;
			var letterColor:uint = _normalLetterColor;
			
			// if no hints then consider tile to be solid
			if ((_horizontalClue == null) && (_verticalClue == null)) {
				if (_isSelected) {
					letterColor = _solidSelectedColor;
					backColor = _solidSelectedColor;
				} else {
					letterColor = _solidColor;
					backColor = _solidColor;
				}
			} else {
				if (_isSelected) {
					letterColor = _selectedLetterColor;
					backColor = _selectedBackColor;
				} else if (_areHintsOn) {
					if (_correctLetter == getLetter())
						letterColor = HINT_CORRECT_COLOR;
					else
						letterColor = HINT_INCORRECT_COLOR;
				}
			}
			setColors(letterColor, backColor);
		}

	}
}
